#pragma rtGlobals = 1		// modern global method

#include <Keyword-Value>
#include <String Substitution>
#include <Strings as Lists>
#include <Readback ModifyStr>

// PPT Boltzmann Fit Control Panel
// Control Panel to fit Boltzmann-like activation kinetics
// part of the PPT, the "Patcher's Power Tools"
//  15.10.97 Dr. Francisco Mendez

// 27.10.97 workaround
// 16.02.98 integrated into the PPT

// ****************************************************

Menu "Macros"
	"Boltzmann Fit", PPTBoltzmannFit ()
end

// ****************************************************

Function PPTBoltzmannGlobals ()
	Silent 1							// initializing . . .

	String oldDataFolder = GetDataFolder (1)
	if (! DataFolderExists ("root:PPTGlobals"))
		NewDataFolder /O root:PPTGlobals
	endif

	SetDataFolder root:PPTGlobals	// create the PPT global variables

	String /G	pptBolBaseName		// BaseName of Waves to show in Popups

	SetDataFolder $oldDataFolder
End Function

// ****************************************************
// Create Analysis Control Panel an place it next to top graph (if there is one)
// ****************************************************

Function PPTBoltzmannFit () : Panel
	Silent 1; PauseUpdate

	Variable	panelX
	variable	panelY
	Variable	panelW = 404
	Variable	panelH = 146

	if (strlen (WinName (0, 1)) <= 0)
		panelX = 10
		panelY = 20
	else
		GetWindow kwTopWin, wsize		// place control panel to the right of the top graph
		panelX = V_right
		panelY = V_top
		if ((panelX + panelW) > 832)	// put horizontal monitor resolution here
			panelX = 832 - panelW
		endif
		if ((panelY + panelH) > 624)	// put vertical monitor resolution here
			panelY = 624 - panelH
		endif
	endif

	PPTBoltzmannGlobals ()
	NewPanel /W=(panelX, panelY, panelX + panelW, panelY + panelH) as "PPT Boltzmann Fit"
	Execute "ModifyPanel cbRGB = (61166, 61166, 61166)"

	// Base Name
	SetVariable	tBaseName,	title = "Base Name"
	SetVariable	tBaseName,	pos = {4, 4}, size = {172, 15}
	SetVariable	tBaseName,	value = root:PPTGlobals:pptBolBaseName

	// Checkbox
	CheckBox	cMatchingWaves,	title="Show matching waves"
	CheckBox	cMatchingWaves,	pos = {4, 22}, size = {172, 20}
	CheckBox	cMatchingWaves,	value = 0

	// Popup Menus
	PopupMenu	pYWave,	title = "Y-Wave"
	PopupMenu	pYWave,	pos = {4, 46}, size = {210, 19}
	PopupMenu pYWave,	value = PPTBolWavePopup ()

	PopupMenu	pXWave,	title = "X-Wave"
	PopupMenu	pXWave,	pos = {4, 70}, size = {210, 19}
	PopupMenu	pXWave,	value = PPTBolWavePopup ()

	PopupMenu	pMWave,	title = "Marker-Wave"
	PopupMenu	pMWave,	pos = {4, 94}, size = {210, 19}
	PopupMenu	pMWave,	value = PPTBolWavePopup ()

	// Button create a Graph
	Button	bShowGraph,	title = "Show Graph"
	Button	bShowGraph,	pos = {4, 118}, size = {84, 24}
	Button	bShowGraph,	proc = PPTShowBoltzmannGraph

	// Button display Table with results
	Button	bShowTable,	title = "Show Table"
	Button	bShowTable,	pos = {92, 118}, size = {84, 24}
	Button	bShowTable,	proc = PPTShowBoltzmannTable

	// Checkboxes which parameters to be fitted
	DrawText 224, 16,"Fit these"
	CheckBox	cYZero,		pos = {224, 20},	size = {72, 20},	title = "Y Zero",		value = 1
 	CheckBox	cYMax,		pos = {224, 40},	size = {72, 20},	title = "Y Max",		value = 1
 	CheckBox	cXZero,		pos = {224, 60},	size = {72, 20},	title = "X Zero",		value = 1
	CheckBox	cXHalf,		pos = {224, 80},	size = {72, 20},	title = "X half",		value = 1
	CheckBox	cXSlope,	pos = {224, 100},	size = {72, 20},	title = "X slope",	value = 1
	CheckBox	cHill,		pos = {224, 120},	size = {72, 20},	title = "Hill",		value = 1

	// Button get zero y from Cursors A and B
	Button	bGetZeroY,		title = "Get from AB"
	Button	bGetZeroY,		pos = {300, 20}, size = {100, 18}
	Button	bGetZeroY,		proc = PPTBolGetZeroY

	// Button get max y from Cursors A and B
	Button	bGetMaxY,		title = "Get from AB"
	Button	bGetMaxY,		pos = {300, 40}, size = {100, 18}
	Button	bGetMaxY,		proc = PPTBolGetMaxY

	// Button get starting x from Cursor A
	Button	bGetZeroX,		title = "Get from A"
	Button	bGetZeroX,		pos = {300, 60}, size = {100, 18}
	Button	bGetZeroX,		proc = PPTBolGetZeroX

	// Button get half x from Cursor A
	Button	bGetHalfX,		title = "Get from A"
	Button	bGetHalfX,		pos = {300, 80}, size = {100, 18}
	Button	bGetHalfX,		proc = PPTBolGetHalfX

	// Button start Fit
	Button	bDoFit,			title = "Do Fit"
	Button	bDoFit,			pos = {300, 112}, size = {100, 30}
	Button	bDoFit,			proc = PPTDoBoltzmannFit
End Function

// ****************************************************
// Control Procedures: get wave names of top graph and fill popup menus
// ****************************************************

Function /S PPTBolWavePopup ()
	SVAR		baseName = root:PPTGlobals:pptBolBaseName
	String		pattern
	Variable	V_Value

	ControlInfo cMatchingWaves
	if (V_Value == 0)					// show all waves
		pattern = "*"
	else									// show only matching waves
		if (strlen (baseName) > 0)	// basename supplied
			pattern = baseName + "*"
		else								// no basename, no more waves
			pattern = ""
		endif
	endif

	 return ("_none_;-;" + WaveList (pattern, ";", ""))
End Function

// ****************************************************
// Create a graph of marker and y-Wave wave vs. x-Wave
// ****************************************************

Function PPTShowBoltzmannGraph (ctrlName) : ButtonControl
	String	ctrlName

	Variable	V_Value
	String		S_Value
	String		yWaveName = ""
	String		xWaveName = ""
	String		mWaveName = ""

	ControlInfo pYWave
	if (V_Value > 2)
		yWaveName = S_Value
	endif
	ControlInfo pXWave
	if (V_Value > 2)
		xWaveName = S_Value
	endif
	ControlInfo pMWave
	if (V_Value > 2)
		mWaveName = S_Value
	endif

	WAVE yWave = $yWaveName
	WAVE xWave = $xWaveName
	WAVE mWave = $mWaveName

	if ((WaveExists (yWave) == 0) %| (WaveExists (xWave) == 0))
		Abort "Please select x- and y-wave!"
	endif

	Display/W=(5,41,408,464)

	Variable axisLen = 1		// 100% if no marker wave
	if (WaveExists (mWave) != 0)
		AppendToGraph /L=l2 mWave vs xWave
		ModifyGraph axisEnab (l2) = {0.7, 1}
		ModifyGraph freePos (l2) = {0, bottom}
		axisLen = 0.65
	endif
	if (WaveExists (yWave) != 0)
		AppendToGraph /L=l1 yWave vs xWave
		ModifyGraph axisEnab (l1) = {0, axisLen}
		ModifyGraph freePos (l1) = {0, bottom}
	endif
	ModifyGraph gaps = 0
	ShowInfo
End Function

// ****************************************************
// Display table with fit-parameters
// ****************************************************

Function PPTShowBoltzmannTable (ctrlName) : ButtonControl
	String		ctrlName

	SVAR		baseName = root:PPTGlobals:pptBolBaseName

	if (cmpstr (baseName, "") == 0)
		Abort "Please define a basename first"
	endif

	// Create wave containing the names of the fit parameters
	if (WaveExists (BoltzmannFitParams) == 0)
		Make/O/N=6/T BoltzmannFitParams
		BoltzmannFitParams [0] = "y zero"
		BoltzmannFitParams [1] = "y max"
		BoltzmannFitParams [2] = "x zero"
		BoltzmannFitParams [3] = "x half"
		BoltzmannFitParams [4] = "x slope"
		BoltzmannFitParams [5] = "Hill-coeff"
	endif

	// Wave containing the fit parameters
	String coeffsWaveName = baseName + "_Coeffs"
	if (WaveExists (coeffsWaveName) == 0)
		Make /N=6/O $coeffsWaveName
		WAVE coeffsWave = $coeffsWaveName
		coeffsWave [4] = 1
		coeffsWave [5] = 1
	endif

	String targetTable = WinName (0, 2)
	if (cmpstr (targetTable, "BoltzmannTable") == 0)
		AppendToTable coeffsWave
	else
		Edit /W=(419,196,785,393) BoltzmannFitParams coeffsWave
		DoWindow/C/T BoltzmannTable,"Boltzmann Analysis Fit Table"
	endif
End Function

// ****************************************************
// set mean value of y-Wave inside cursors as zero-value
// ****************************************************

Function PPTBolGetZeroY (ctrlName) : ButtonControl
	String		ctrlName

	SVAR		baseName = root:PPTGlobals:pptBolBaseName
	String		coeffsWaveName, yWaveName
	Variable	V_Value
	String		S_Value

	ControlInfo pYWave
	if (V_Value < 3)
		Abort "Please select an y-Wave first!"
	else
		yWaveName = S_Value
	endif
	if ((strlen (CsrWave (A)) <= 0) %| (strlen (CsrWave (B)) <= 0))
		Abort "Please set both cursors (A and B)!"
	endif

	WAVE yWave = $yWaveName
	WaveStats/Q/R=(xcsr(A),xcsr(B)) yWave

	coeffsWaveName = baseName + "_Coeffs"
	if (WaveExists ($coeffsWaveName) == 0)
		PPTShowBoltzmannTable ("")		// create Coeffs-Wave
	endif

	WAVE coeffsWave = $coeffsWaveName
	coeffsWave [0] = V_avg
End Function

// ****************************************************
// set mean value of y-Wave inside cursors as max-value
// ****************************************************

Function PPTBolGetMaxY (ctrlName) : ButtonControl
	String		ctrlName

	SVAR		baseName = root:PPTGlobals:pptBolBaseName
	String		coeffsWaveName, yWaveName
	Variable	V_Value
	String		S_Value

	ControlInfo pYWave
	if (V_Value < 3)
		Abort "Please select an y-Wave first!"
	else
		yWaveName = S_Value
	endif
	if ((strlen (CsrWave (A)) <= 0) %| (strlen (CsrWave (B)) <= 0))
		Abort "Please set both cursors (A and B)!"
	endif

	WAVE yWave = $yWaveName
	WaveStats/Q/R=(xcsr(A),xcsr(B)) yWave

	coeffsWaveName = baseName + "_Coeffs"
	if (WaveExists ($coeffsWaveName) == 0)
		PPTShowBoltzmannTable ("")		// create Coeffs-Wave
	endif

	WAVE coeffsWave = $coeffsWaveName
	coeffsWave [1] = V_avg
End Function

// ****************************************************
// Define starting x value using position of cursor A
// ****************************************************

Function PPTBolGetZeroX (ctrlName) : ButtonControl
	String		ctrlName

	SVAR		baseName = root:PPTGlobals:pptBolBaseName
	String		coeffsWaveName

	if (strlen (CsrWave (A)) <= 0)
		Abort "Please set the cursor A!"
	endif

	coeffsWaveName = baseName + "_Coeffs"
	if (WaveExists ($coeffsWaveName) == 0)
		PPTShowBoltzmannTable ("")		// create Coeffs-Wave
	endif

	WAVE coeffsWave = $coeffsWaveName
	coeffsWave [2] = hcsr (A)
End Function

// ****************************************************
// Define half x value using position of cursor A
// ****************************************************

Function PPTBolGetHalfX (ctrlName) : ButtonControl
	String		ctrlName

	SVAR		baseName = root:PPTGlobals:pptBolBaseName
	String		coeffsWaveName

	if (strlen (CsrWave (A)) <= 0)
		Abort "Please set the cursor A!"
	endif

	coeffsWaveName = baseName + "_Coeffs"
	if (WaveExists ($coeffsWaveName) == 0)
		PPTShowBoltzmannTable ("")		// create Coeffs-Wave
	endif

	WAVE coeffsWave = $coeffsWaveName
	coeffsWave [3] = hcsr (A)
End Function

// ****************************************************
// Do the fit
// ****************************************************

Function PPTDoBoltzmannFit (ctrlName) : ButtonControl
	String 		ctrlName

	SVAR		baseName = root:PPTGlobals:pptBolBaseName
	String		coeffsWaveName, xWaveName, yWaveName
	String		S_Value
	Variable	V_Value

	if ((strlen (CsrWave (A)) <= 0) %| (strlen (CsrWave (B)) <= 0))
		Abort "Please set the cursors first!"
	endif

	ControlInfo pYWave
	if (V_Value < 3)
		Abort "Please select a y-Wave wave first!"
	else
		yWaveName = S_Value
	endif
	ControlInfo pXWave
	if (V_Value < 3)
		Abort "Please select an x-wave first!"
	else
		xWaveName = S_Value
	endif

	if (cmpstr (xWaveName, yWaveName) == 0)
		Abort "X- and y-wave have to be different!"
	endif

	coeffsWaveName = baseName + "_Coeffs"
	WAVE coeffsWave = $coeffsWaveName
	if (WaveExists (coeffsWave) == 0)
		PPTShowBoltzmannTable ("")
	endif
	WAVE yWave = $yWaveName
	WAVE xWave = $xWaveName

	// Check, which parameters to be fitted and do the fit
	String holdStr
	holdStr = PPTBolCheckToBoolStr ("cYZero")		// Y Zero
	holdStr += PPTBolCheckToBoolStr ("cYMax")	// Y Max
	holdStr += PPTBolCheckToBoolStr ("cXZero")	// X Zero
	holdStr += PPTBolCheckToBoolStr ("cXHalf")	// X Half
	holdStr += PPTBolCheckToBoolStr ("cXSlope")	// X Slope
	holdStr += PPTBolCheckToBoolStr ("cHill")		// Hill
	FuncFit/Q/H=(holdStr) PPTBoltzmannFunc coeffsWave yWave (xcsr(A),xcsr(B)) /X=xWave /D

	// Create dependency to modify fit paramenters interactively
	String theCommand
	sprintf theCommand, "fit_%s := %s[0] + (%s[1] - %s[0]) * (1 / (1 + exp (-((x - %s[2]) - %s[3]) / %s[4]))^%s[5])", yWaveName, coeffsWaveName, coeffsWaveName, coeffsWaveName, coeffsWaveName, coeffsWaveName, coeffsWaveName, coeffsWaveName
	Execute theCommand
End Function

// ****************************************************

Function /S PPTBolCheckToBoolStr (ctrlName)
	String		ctrlName
	Variable	V_Value

	ControlInfo $ctrlName
	if (V_Value)
		return ("0")
	else
		return ("1")
	endif
End Function

// ****************************************************
// Fit Function for Boltzmann Activation
// ****************************************************

Function PPTBoltzmannFunc (w, x)		
	Wave		w
	Variable	x	
				
	Variable	yZero	= w [0]
	Variable	yMax	= w [1]
	Variable	xZero	= w [2]
	Variable	xHalf	= w [3]
	Variable	xSlope	= w [4]
	Variable	Hill		= w [5]
	
	return (yZero + (yMax - yZero) * (1 / (1 + exp (-((x - xZero) - xHalf) / xSlope)))^Hill)
End Function
